@@ -6,10 +6,9 @@ from glymur import version
 __version__ = version.version
 
 from .jp2k import Jp2k
-from .jp2box import (get_printoptions,
-                     set_printoptions,
-                     get_parseoptions,
-                     set_parseoptions)
+from .config import (get_option, set_option, reset_option,
+                     get_printoptions, set_printoptions,
+                     get_parseoptions, set_parseoptions)
 
 from . import data
 
@@ -22,4 +21,5 @@ def runtests():
 
 
 __all__ = [__version__, Jp2k, get_printoptions, set_printoptions,
-           get_parseoptions, set_parseoptions, data, runtests]
+           get_parseoptions, set_parseoptions, get_option, set_option,
+           reset_option, data, runtests]
