@@ -6,10 +6,18 @@ Copyright 2013 John Evans
 
 License:  MIT
 """
-
+# Standard library imports...
+from collections import Counter
+import ctypes
+import math
+import os
+import re
+import struct
 import sys
+from uuid import UUID
+import warnings
 
-# Exitstack not found in contextlib in 2.7
+# Standard library, third party imports...
 if sys.hexversion >= 0x03030000:
     from contextlib import ExitStack
     from itertools import filterfalse
@@ -17,17 +25,9 @@ else:
     from contextlib2 import ExitStack
     from itertools import ifilterfalse as filterfalse
 
-from collections import Counter
-import ctypes
-import math
-import os
-import re
-import struct
-from uuid import UUID
-import warnings
-
 import numpy as np
 
+# Local imports...
 from .codestream import Codestream
 from . import core, version
 from .jp2box import (Jp2kBox, JPEG2000SignatureBox, FileTypeBox,
@@ -53,15 +53,15 @@ class Jp2k(Jp2kBox):
     Properties
     ----------
     ignore_pclr_cmap_cdef : bool
-        whether or not to ignore the pclr, cmap, or cdef boxes during any
+        Whether or not to ignore the pclr, cmap, or cdef boxes during any
         color transformation, defaults to False.
     layer : int
-        zero-based number of quality layer to decode
+        Zero-based number of quality layer to decode.
     verbose : bool
-        whether or not to print informational messages produced by the
-        OpenJPEG library, defaults to false
-    codestream : object
-        JP2 or J2K codestream object
+        Whether or not to print informational messages produced by the
+        OpenJPEG library, defaults to false.
+    codestream : glymur.codestream.Codestream
+        JP2 or J2K codestream object.
 
     Examples
     --------
@@ -88,31 +88,31 @@ class Jp2k(Jp2kBox):
 
         Parameters
         ----------
-        filename : str or file
-            the path to JPEG 2000 file
+        filename : str
+            The path to JPEG 2000 file.
         image_data : ndarray, optional
-            image data to be written
-        shape : tuple
-            size of image data, only required when image_data is not provided
+            Image data to be written to file.
+        shape : tuple, optional
+            Size of image data, only required when image_data is not provided.
         cbsize : tuple, optional
-            code block size (DY, DX)
+            Code block size (NROWS, NCOLS)
         cinema2k : int, optional
-            frames per second, either 24 or 48
+            Frames per second, either 24 or 48.
         cinema4k : bool, optional
-            set to True to specify Cinema4K mode, defaults to false
-        colorspace : str, optional
-            either 'rgb' or 'gray'
-        cratios : iterable
-            compression ratios for successive layers
+            Set to True to specify Cinema4K mode, defaults to false.
+        colorspace : {'rgb', 'gray'}
+            The image color space.
+        cratios : iterable, optional
+            Compression ratios for successive layers.
         eph : bool, optional
-            if true, write SOP marker after each header packet
+            If true, write SOP marker after each header packet.
         grid_offset : tuple, optional
-            offset (DY, DX) of the origin of the image in the reference grid
+            Offset (DY, DX) of the origin of the image in the reference grid.
         irreversible : bool, optional
-            if true, use the irreversible DWT 9-7 transform
+            If true, use the irreversible DWT 9-7 transform.
         mct : bool, optional
-            specifies usage of the multi component transform, if not
-            specified, defaults to True if the colorspace is RGB
+            Usage of the multi component transform.  If not specified, defaults
+            to True if the color space is RGB.
         modesw : int, optional
             mode switch
                 1 = BYPASS(LAZY)
@@ -122,23 +122,22 @@ class Jp2k(Jp2kBox):
                 16 = ERTERM(SEGTERM)
                 32 = SEGMARK(SEGSYM)
         numres : int, optional
-            number of resolutions
-        prog : str, optional
-            progression order, one of "LRCP" "RLCP", "RPCL", "PCRL", "CPRL"
+            Number of resolutions.
+        prog : {"LRCP" "RLCP", "RPCL", "PCRL", "CPRL"}
+            Progression order.
         psnr : iterable, optional
-            different PSNR for successive layers
+            Different PSNR for successive layers.
         psizes : list, optional
-            list of precinct sizes, each precinct size tuple is defined in
-            (height x width)
+            List of precinct sizes, each precinct size tuple is defined in
+            (height x width).
         sop : bool, optional
-            if true, write SOP marker before each packet
+            If true, write SOP marker before each packet.
         subsam : tuple, optional
-            subsampling factors (dy, dx)
+            Subsampling factors (dy, dx).
         tilesize : tuple, optional
-            numeric tuple specifying tile size in terms of (numrows, numcols),
-            not (X, Y)
+            Tile size in terms of (numrows, numcols), not (X, Y).
         verbose : bool, optional
-            print informational messages produced by the OpenJPEG library
+            Print informational messages produced by the OpenJPEG library.
         """
         Jp2kBox.__init__(self)
         self.filename = filename
@@ -200,6 +199,15 @@ class Jp2k(Jp2kBox):
 
     @verbose.setter
     def verbose(self, verbose):
+        """Verbosity property.
+
+        If True, print informational messages from the OPENJPEG library.
+
+        Parameters
+        ----------
+        verbose : {True, False}
+            Set to verbose or not.
+        """
         self._verbose = verbose
 
     @property
@@ -323,10 +331,10 @@ class Jp2k(Jp2kBox):
         params : ctypes struct
             Corresponds to compression parameters structure used by the
             library.
-        cinema_mode : str
-            Either 'cinema2k' or 'cinema4k'
-        fps : int
-            Frames per second, should be either 24 or 48.
+        cinema_mode : {'cinema2k', 'cinema4k}
+            Use either Cinema2K or Cinema4K profile.
+        fps : {24, 48}
+            Frames per second.
         """
         if re.match("1.5|2.0.0", version.openjpeg_version) is not None:
             msg = ("Writing Cinema2K or Cinema4K files is not supported with "
@@ -379,9 +387,9 @@ class Jp2k(Jp2kBox):
         Parameters
         ----------
         img_array : ndarray
-            image data to be written to file
+            Image data to be written to file.
         kwargs : dictionary
-            non-image keyword inputs provided to write method
+            Non-image keyword inputs provided to write method.
         """
         other_args = (mct, cratios, psnr, irreversible, cbsize, eph,
                       grid_offset, modesw, numres, prog, psizes, sop, subsam)
@@ -484,8 +492,8 @@ class Jp2k(Jp2kBox):
             cparams.tcp_mct = 1 if self._colorspace == opj2.CLRSPC_SRGB else 0
         else:
             if self._colorspace == opj2.CLRSPC_GRAY:
-                msg = "Cannot specify usage of the multi component transform "
-                msg += "if the colorspace is gray."
+                msg = ("Cannot specify usage of the multi component transform "
+                       "if the colorspace is gray.")
                 raise IOError(msg)
             cparams.tcp_mct = 1 if mct else 0
 
@@ -501,8 +509,8 @@ class Jp2k(Jp2kBox):
         in memory.
         """
         if re.match("0|1.[0-4]", version.openjpeg_version) is not None:
-            msg = "You must have at least version 1.5 of OpenJPEG "
-            msg += "in order to write images."
+            msg = ("You must have at least version 1.5 of OpenJPEG "
+                   "in order to write images.")
             raise RuntimeError(msg)
 
         self._determine_colorspace(**kwargs)
@@ -585,8 +593,7 @@ class Jp2k(Jp2kBox):
         Cannot specify a colorspace with J2K.
         """
         if cparams.codec_fmt == opj2.CODEC_J2K and colorspace is not None:
-            msg = 'Do not specify a colorspace when writing a raw '
-            msg += 'codestream.'
+            msg = 'Do not specify a colorspace when writing a raw codestream.'
             raise IOError(msg)
 
     def _validate_codeblock_size(self, cparams):
@@ -675,8 +682,8 @@ class Jp2k(Jp2kBox):
         Only uint8 and uint16 images are currently supported.
         """
         if img_array.dtype != np.uint8 and img_array.dtype != np.uint16:
-            msg = "Only uint8 and uint16 datatypes are currently supported "
-            msg += "when writing."
+            msg = ("Only uint8 and uint16 datatypes are currently supported "
+                   "when writing.")
             raise RuntimeError(msg)
 
     def _validate_compression_params(self, img_array, cparams, colorspace):
@@ -798,8 +805,8 @@ class Jp2k(Jp2kBox):
         if not ((box.box_id == 'xml ') or
                 (box.box_id == 'uuid' and
                  box.uuid == UUID('be7acfcb-97a9-42e8-9c71-999491e3afac'))):
-            msg = "Only XML boxes and XMP UUID boxes can currently be "
-            msg += "appended."
+            msg = ("Only XML boxes and XMP UUID boxes can currently be "
+                   "appended.")
             raise IOError(msg)
 
         # Check the last box.  If the length field is zero, then rewrite
@@ -889,9 +896,9 @@ class Jp2k(Jp2kBox):
         offset = box.offset
         if offset == -1:
             if self.box[1].brand == 'jpx ':
-                msg = "The codestream box must have its offset and "
-                msg += "length attributes fully specified if the file "
-                msg += "type brand is JPX."
+                msg = ("The codestream box must have its offset and length "
+                       "attributes fully specified if the file type brand is "
+                       "JPX.")
                 raise IOError(msg)
 
             # Find the first codestream in the file.
@@ -1735,8 +1742,8 @@ class Jp2k(Jp2kBox):
             count = self._collect_box_count(boxes)
             for box_id in count.keys():
                 if box_id not in JP2_IDS:
-                    msg = "The presence of a '{0}' box requires that the file "
-                    msg += "type brand be set to 'jpx '."
+                    msg = ("The presence of a '{0}' box requires that the "
+                           "file type brand be set to 'jpx '.")
                     raise IOError(msg.format(box_id))
 
             self._validate_jp2_colr(boxes)
@@ -1749,8 +1756,8 @@ class Jp2k(Jp2kBox):
         jp2h = lst[0]
         for colr in [box for box in jp2h.box if box.box_id == 'colr']:
             if colr.approximation != 0:
-                msg = "A JP2 colr box cannot have a non-zero approximation "
-                msg += "field."
+                msg = ("A JP2 colr box cannot have a non-zero approximation "
+                       "field.")
                 raise IOError(msg)
 
     def _validate_jpx_box_sequence(self, boxes):
@@ -1766,8 +1773,8 @@ class Jp2k(Jp2kBox):
         # Check for a bad sequence of boxes.
         # 1st two boxes must be 'jP  ' and 'ftyp'
         if boxes[0].box_id != 'jP  ' or boxes[1].box_id != 'ftyp':
-            msg = "The first box must be the signature box and the second "
-            msg += "must be the file type box."
+            msg = ("The first box must be the signature box and the second "
+                   "must be the file type box.")
             raise IOError(msg)
 
         # The compatibility list must contain at a minimum 'jp2 '.
@@ -1784,8 +1791,8 @@ class Jp2k(Jp2kBox):
         jp2c_lst = [idx for (idx, box) in enumerate(boxes)
                     if box.box_id == 'jp2c']
         if len(jp2c_lst) == 0:
-            msg = "A codestream box must be defined in the outermost "
-            msg += "list of boxes."
+            msg = ("A codestream box must be defined in the outermost "
+                   "list of boxes.")
             raise IOError(msg)
 
         jp2c_idx = jp2c_lst[0]
@@ -1807,8 +1814,8 @@ class Jp2k(Jp2kBox):
 
         # 1st jp2 header box must be ihdr
         if jp2h.box[0].box_id != 'ihdr':
-            msg = "The first box in the jp2 header box must be the image "
-            msg += "header box."
+            msg = ("The first box in the jp2 header box must be the image "
+                   "header box.")
             raise IOError(msg)
 
         # colr must be present in jp2 header box.
@@ -1826,21 +1833,21 @@ class Jp2k(Jp2kBox):
         cdef_lst = [j for (j, box) in enumerate(jp2h.box)
                     if box.box_id == 'cdef']
         if len(cdef_lst) > 1:
-            msg = "Only one channel definition box is allowed in the "
-            msg += "JP2 header."
+            msg = ("Only one channel definition box is allowed in the "
+                   "JP2 header.")
             raise IOError(msg)
         elif len(cdef_lst) == 1:
             cdef = jp2h.box[cdef_lst[0]]
             if colr.colorspace == core.SRGB:
                 if any([chan + 1 not in cdef.association or
                         cdef.channel_type[chan] != 0 for chan in [0, 1, 2]]):
-                    msg = "All color channels must be defined in the "
-                    msg += "channel definition box."
+                    msg = ("All color channels must be defined in the "
+                           "channel definition box.")
                     raise IOError(msg)
             elif colr.colorspace == core.GREYSCALE:
                 if 0 not in cdef.channel_type:
-                    msg = "All color channels must be defined in the "
-                    msg += "channel definition box."
+                    msg = ("All color channels must be defined in the "
+                           "channel definition box.")
                     raise IOError(msg)
 
     def _check_jp2h_child_boxes(self, boxes, parent_box_name):
@@ -1900,8 +1907,8 @@ class Jp2k(Jp2kBox):
 
         # If there is one data reference box, then there must also be one ftbl.
         if 'dtbl' in count and 'ftbl' not in count:
-            msg = 'The presence of a data reference box requires the presence '
-            msg += 'of a fragment table box as well.'
+            msg = ('The presence of a data reference box requires the '
+                   'presence of a fragment table box as well.')
             raise IOError(msg)
 
     def _validate_singletons(self, boxes):
@@ -1920,8 +1927,8 @@ class Jp2k(Jp2kBox):
         for box in boxes:
             if box.box_id in JPX_IDS:
                 if brand != 'jpx ':
-                    msg = "A JPX box requires that the file type box brand be "
-                    msg += "'jpx '."
+                    msg = ("A JPX box requires that the file type box brand "
+                           "be 'jpx '.")
                     raise RuntimeError(msg)
             if hasattr(box, 'box') != 0:
                 # Same set of checks on any child boxes.
@@ -1937,8 +1944,8 @@ class Jp2k(Jp2kBox):
         for box in boxes:
             if box.box_id in JPX_IDS:
                 if len(set(['jpx ', 'jpxb']).intersection(jpx_cl)) == 0:
-                    msg = "A JPX box requires that either 'jpx ' or 'jpxb' be "
-                    msg += "present in the ftype compatibility list."
+                    msg = ("A JPX box requires that either 'jpx ' or 'jpxb' "
+                           "be present in the ftype compatibility list.")
                     raise RuntimeError(msg)
             if hasattr(box, 'box') != 0:
                 # Same set of checks on any child boxes.
@@ -1954,8 +1961,8 @@ class Jp2k(Jp2kBox):
                 if hasattr(box, 'box'):
                     for boxi in box.box:
                         if boxi.box_id == 'lbl ':
-                            msg = "A label box cannot be nested inside a "
-                            msg += "{0} box."
+                            msg = ("A label box cannot be nested inside a "
+                                   "{0} box.")
                             msg = msg.format(box.box_id)
                             raise IOError(msg)
                     # Same set of checks on any child boxes.
