@@ -2,8 +2,10 @@
 """
 Test suite for printing.
 """
+# Standard library imports ...
 from io import BytesIO
 import os
+import pkg_resources as pkg
 import re
 import struct
 import sys
@@ -22,8 +24,13 @@ if sys.hexversion <= 0x03030000:
 else:
     from unittest.mock import patch
 
+# Third party imports ...
 import numpy as np
-import lxml.etree as ET
+
+try:
+    import lxml.etree as ET
+except ImportError:
+    import xml.etree.ElementTree as ET
 
 import glymur
 from glymur.core import RESTRICTED_ICC_PROFILE, ANY_ICC_PROFILE
@@ -38,7 +45,9 @@ from .fixtures import (WINDOWS_TMP_FILE_MSG,
 
 @unittest.skipIf(os.name == "nt", WINDOWS_TMP_FILE_MSG)
 class TestPrinting(unittest.TestCase):
-    """Tests for verifying how printing works."""
+    """
+    Tests for verifying how printing works.
+    """
     def setUp(self):
         self.jpxfile = glymur.data.jpxfile()
         self.jp2file = glymur.data.nemo()
@@ -50,6 +59,17 @@ class TestPrinting(unittest.TestCase):
     def tearDown(self):
         glymur.reset_option('all')
 
+    def test_bad_color_specification(self):
+        """
+        Invalid channel type should not prevent printing.
+        """
+        relfile = os.path.join('data', 'issue392.jp2')
+        file = pkg.resource_filename(__name__, relfile)
+        with warnings.catch_warnings():
+            warnings.simplefilter('ignore')
+            jp2 = Jp2k(file)
+        str(jp2)
+
     def test_palette(self):
         """
         verify printing of pclr box
@@ -102,6 +122,7 @@ class TestPrinting(unittest.TestCase):
                     '    Channel 2 (color) ==> (1)')
         self.assertEqual(actual, expected)
 
+    @unittest.skipIf('lxml' not in sys.modules.keys(), "No lxml")
     def test_xml(self):
         """
         verify printing of XML box
@@ -255,6 +276,7 @@ class TestPrinting(unittest.TestCase):
         with self.assertRaises(KeyError):
             glymur.set_option('hi', 'low')
 
+    @unittest.skipIf('lxml' not in sys.modules.keys(), "No lxml")
     @unittest.skipIf(re.match("1.5|2",
                               glymur.version.openjpeg_version) is None,
                      "Must have openjpeg 1.5 or higher to run")
@@ -449,12 +471,17 @@ class TestPrinting(unittest.TestCase):
         self.assertEqual(actual, expected)
 
     def test_xmp(self):
-        """Verify the printing of a UUID/XMP box."""
+        """
+        Verify the printing of a UUID/XMP box.
+
+        If no lxml, just verify that we don't error out.
+        """
         j = glymur.Jp2k(self.jp2file)
         actual = str(j.box[3])
 
-        expected = fixtures.nemo_xmp_box
-        self.assertEqual(actual, expected)
+        if 'lxml' in sys.modules.keys():
+            expected = fixtures.nemo_xmp_box
+            self.assertEqual(actual, expected)
 
     def test_codestream(self):
         """
@@ -504,6 +531,7 @@ class TestPrinting(unittest.TestCase):
                '        "Created by OpenJPEG version 2.0.0"')
         self.assertEqual(actual, exp)
 
+    @unittest.skipIf('lxml' not in sys.modules.keys(), "No lxml")
     @unittest.skipIf(sys.hexversion < 0x03000000,
                      "Only trusting python3 for printing non-ascii chars")
     def test_xml_latin1(self):
@@ -529,6 +557,7 @@ class TestPrinting(unittest.TestCase):
                         "    <flow>Strömung</flow>")
         self.assertEqual(actual, expected)
 
+    @unittest.skipIf('lxml' not in sys.modules.keys(), "No lxml")
     @unittest.skipIf(sys.hexversion < 0x03000000,
                      "Only trusting python3 for printing non-ascii chars")
     def test_xml_cyrrilic(self):
@@ -931,6 +960,7 @@ class TestPrinting(unittest.TestCase):
         actual = str(box)
         self.assertEqual(actual, fixtures.text_GBR_rreq)
 
+    @unittest.skipIf('lxml' not in sys.modules.keys(), "No lxml")
     def test_bom(self):
         """
         Byte order markers are illegal in UTF-8.  Issue 185
@@ -1024,11 +1054,11 @@ class TestPrinting(unittest.TestCase):
         with warnings.catch_warnings():
             warnings.simplefilter('ignore')
             glymur.set_printoptions(short=True)
-        with patch('sys.stdout', new=StringIO()) as fake_out:
-            print(jp2)
-            actual = fake_out.getvalue().strip()
-            # Get rid of the file line, that's kind of volatile.
-            actual = '\n'.join(actual.splitlines()[1:])
+
+        actual = str(jp2)
+
+        # Get rid of leading "File" line, as that is volatile.
+        actual = '\n'.join(actual.splitlines()[1:])
 
         expected = fixtures.nemo_dump_short
         self.assertEqual(actual, expected)
@@ -1046,11 +1076,11 @@ class TestPrinting(unittest.TestCase):
         with warnings.catch_warnings():
             warnings.simplefilter('ignore')
             glymur.set_printoptions(xml=False)
-        with patch('sys.stdout', new=StringIO()) as fake_out:
-            print(jp2)
-            actual = fake_out.getvalue().strip()
-            # Get rid of the file line, that's kind of volatile.
-            actual = '\n'.join(actual.splitlines()[1:])
+
+        actual = str(jp2)
+
+        # Get rid of leading "File" line, as that is volatile.
+        actual = '\n'.join(actual.splitlines()[1:])
 
         # shave off the XML and non-main-header segments
         expected = fixtures.nemo_dump_no_xml
@@ -1067,11 +1097,11 @@ class TestPrinting(unittest.TestCase):
         """
         jp2 = Jp2k(self.jp2file)
         glymur.set_option('print.xml', False)
-        with patch('sys.stdout', new=StringIO()) as fake_out:
-            print(jp2)
-            actual = fake_out.getvalue().strip()
-            # Get rid of the file line, that's kind of volatile.
-            actual = '\n'.join(actual.splitlines()[1:])
+
+        actual = str(jp2)
+
+        # Get rid of the file line, that's kind of volatile.
+        actual = '\n'.join(actual.splitlines()[1:])
 
         # shave off the XML and non-main-header segments
         expected = fixtures.nemo_dump_no_xml
@@ -1080,6 +1110,7 @@ class TestPrinting(unittest.TestCase):
         opt = glymur.get_option('print.xml')
         self.assertFalse(opt)
 
+    @unittest.skipIf('lxml' not in sys.modules.keys(), "No lxml")
     def test_suppress_codestream_old_option(self):
         """
         Verify printing with codestream suppressed, deprecated
@@ -1088,11 +1119,11 @@ class TestPrinting(unittest.TestCase):
         with warnings.catch_warnings():
             warnings.simplefilter('ignore')
             glymur.set_printoptions(codestream=False)
-        with patch('sys.stdout', new=StringIO()) as fake_out:
-            print(jp2)
-            actual = fake_out.getvalue().strip()
-            # Get rid of the file line, that's kind of volatile.
-            actual = '\n'.join(actual.splitlines()[1:])
+
+        actual = str(jp2)
+
+        # Get rid of the file line, that's kind of volatile.
+        actual = '\n'.join(actual.splitlines()[1:])
 
         expected = fixtures.nemo_dump_no_codestream
         self.assertEqual(actual, expected)
@@ -1102,6 +1133,7 @@ class TestPrinting(unittest.TestCase):
             opt = glymur.get_printoptions()['codestream']
         self.assertFalse(opt)
 
+    @unittest.skipIf('lxml' not in sys.modules.keys(), "No lxml")
     def test_suppress_codestream(self):
         """
         Verify printing with codestream suppressed
@@ -1118,6 +1150,7 @@ class TestPrinting(unittest.TestCase):
         opt = glymur.get_option('print.codestream')
         self.assertFalse(opt)
 
+    @unittest.skipIf('lxml' not in sys.modules.keys(), "No lxml")
     def test_full_codestream(self):
         """
         Verify printing with the full blown codestream
@@ -1134,6 +1167,7 @@ class TestPrinting(unittest.TestCase):
         opt = glymur.get_option('print.codestream')
         self.assertTrue(opt)
 
+
 class TestJp2dump(unittest.TestCase):
     """Tests for verifying how jp2dump console script works."""
     def setUp(self):
@@ -1160,6 +1194,10 @@ class TestJp2dump(unittest.TestCase):
     def test_default_nemo(self):
         """by default one should get the main header"""
         actual = self.run_jp2dump(['', self.jp2file])
+        if 'lxml' not in sys.modules.keys():
+            # No lxml, so don't bother verifying.  We know at least that it
+            # runs.
+            return
 
         # shave off the  non-main-header segments
         lines = fixtures.nemo.split('\n')
@@ -1167,6 +1205,7 @@ class TestJp2dump(unittest.TestCase):
         expected = '\n'.join(expected)
         self.assertEqual(actual, expected)
 
+    @unittest.skipIf('lxml' not in sys.modules.keys(), "No lxml")
     def test_jp2_codestream_0(self):
         """Verify dumping with -c 0, supressing all codestream details."""
         actual = self.run_jp2dump(['', '-c', '0', self.jp2file])
@@ -1177,6 +1216,7 @@ class TestJp2dump(unittest.TestCase):
         expected = '\n'.join(expected)
         self.assertEqual(actual, expected)
 
+    @unittest.skipIf('lxml' not in sys.modules.keys(), "No lxml")
     def test_jp2_codestream_1(self):
         """Verify dumping with -c 1, print just the header."""
         actual = self.run_jp2dump(['', '-c', '1', self.jp2file])
@@ -1185,9 +1225,9 @@ class TestJp2dump(unittest.TestCase):
         lines = fixtures.nemo.split('\n')
         expected = lines[0:140]
         expected = '\n'.join(expected)
-        self.maxDiff = None
         self.assertEqual(actual, expected)
 
+    @unittest.skipIf('lxml' not in sys.modules.keys(), "No lxml")
     def test_jp2_codestream_2(self):
         """Verify dumping with -c 2, print entire jp2 jacket, codestream."""
         actual = self.run_jp2dump(['', '-c', '2', self.jp2file])
@@ -1225,6 +1265,7 @@ class TestJp2dump(unittest.TestCase):
 
         self.assertEqual(actual, fixtures.nemo_dump_short)
 
+    @unittest.skipIf('lxml' not in sys.modules.keys(), "No lxml")
     def test_suppress_xml(self):
         """Verify dumping with -x, suppress XML."""
         actual = self.run_jp2dump(['', '-x', self.jp2file])
@@ -1234,5 +1275,4 @@ class TestJp2dump(unittest.TestCase):
         expected = lines[0:18]
         expected.extend(lines[104:140])
         expected = '\n'.join(expected)
-        self.maxDiff = None
         self.assertEqual(actual, expected)
