@@ -11,14 +11,18 @@ License:  MIT
 import sys
 from distutils.version import LooseVersion
 
-import lxml.etree
+try:
+    import lxml.etree
+except ImportError:
+    pass
+
 import numpy as np
 
 from .lib import openjpeg as opj, openjp2 as opj2
 
 # Do not change the format of this next line!  Doing so risks breaking
 # setup.py
-version = "0.8.4"
+version = "0.8.6"
 _sv = LooseVersion(version)
 version_tuple = _sv.version
 
@@ -49,12 +53,20 @@ OPENJPEG      {openjpeg}
 Python        {python}
 sys.platform  {platform}
 sys.maxsize   {maxsize}
-lxml          {elxml}
 numpy         {numpy}
-""".format(glymur=version,
-           openjpeg=openjpeg_version,
-           python=sys.version,
-           platform=sys.platform,
-           maxsize=sys.maxsize,
-           elxml=lxml.etree.__version__,
-           numpy=np.__version__)
+"""
+
+kwargs = {
+    'glymur': version,
+    'openjpeg': openjpeg_version,
+    'python': sys.version,
+    'platform': sys.platform,
+    'maxsize': sys.maxsize,
+    'numpy': np.__version__,
+}
+
+if 'lxml' in sys.modules.keys():
+    info += "lxml          {elxml}\n"
+    kwargs['elxml'] = lxml.etree.__version__
+
+info = info.format(**kwargs)
